;; scss.lex: Lexer definition for SCSS
;; Copyright (C) 2005 Julian Graham

;; SCSS is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; SCSS is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with SCSS; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

unicode 	\\[0-9a-f]{1,6}(\013\n|[ \n\013\009\012])?
escape 		{unicode}|\\[ -~\200-\4177777]
nonascii	[^\0-\127]
nmstart 	[_a-zA-Z]|{nonascii}|{escape}
nmchar 		[_a-zA-Z0-9\-]|{nonascii}|{escape}
ident 		[-]?{nmstart}{nmchar}*
name 		{nmchar}+
num 		[0-9]+|[0-9]*\.[0-9]+
nl 		\n|\013\n|\013|\012
string1 	\"([\011 !#$%&(-~]|\\{nl}|\'|{nonascii}|{escape})*\"
string2 	\'([\011 !#$%&(-~]|\\{nl}|\"|{nonascii}|{escape})*\'
string 		{string1}|{string2}
w 		[ \009\013\n\012]*

%%

{ident}		(cons 'IDENT yytext)
@{ident}	(cons 'ATKEYWORD yytext)
{string}	(cons 'STRING yytext)
#{name}		(cons 'HASH yytext)
{num}		(cons 'NUMBER yytext)
{num}%		(cons 'PERCENTAGE yytext)
{num}{ident}	(cons 'DIMENSION yytext)
url\({w}{string}{w}\)|url\({w}([!#$%&*-~]|{nonascii}|{escape})*{w}\)	(cons 'URI yytext)
U\+[0-9A-F?]{1,6}(-[0-9A-F]{1,6})?	(cons 'UNICODE-RANGE yytext)
<!--		(list 'CDO)
-->		(list 'CDC)
";"		(list 'SEMICOLON)
\{		(list 'LBRACE)
\}		(list 'RBRACE)
\(		(list 'LPAREN)
\)		(list 'RPAREN)
\[		(list 'LBRACK)
\]		(list 'RBRACK)
\/\*[^*]*\*+([^/*][^*]*\*+)*\/
{ident}\(	(cons 'FUNCTION yytext)
~\=		(list 'INCLUDES)
\|\=		(list 'DASHMATCH)
\=		(list 'EXACTMATCH)
[ \n\013\009\012]+   	(list 'S)
.		(cons 'DELIM yytext)
<<EOF>>         (list '*eoi*)
<<ERROR>>       (begin (display "ERROR: ") (display yytext) (newline))
