;; scss.y: LALR parser definition for SCSS
;; Copyright (C) 2005 Julian Graham

;; SCSS is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; SCSS is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with SCSS; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(lalr-parser

  (output: scss-parser "parser.scm")
 
  (IDENT ATKEYWORD STRING HASH NUMBER PERCENTAGE DIMENSION URI UNICODE-RANGE
   CDO CDC SEMICOLON LBRACE RBRACE LPAREN RPAREN LBRACK RBRACK S FUNCTION 
   EXACTMATCH INCLUDES DASHMATCH DELIM)

  (stylesheet (stylesheet-single)            : (list $1)
	      (stylesheet-single stylesheet) : (null-merge $1 $2))
  (stylesheet-single (CDO)       : (list)
                     (CDC)       : (list)
                     (S)         : (list)      
		     (statement) : $1)
  (statement (ruleset) : $1
	     (at-rule) : $1)
  
  (at-rule (ATKEYWORD block-or-s-semi)                 : $1
	   (ATKEYWORD s-plus block-or-s-semi)          : $1
	   (ATKEYWORD any-plus block-or-s-semi)        : $1
	   (ATKEYWORD s-plus any-plus block-or-s-semi) : $1)
  
  (block (LBRACE RBRACE)                                : (list)
         (LBRACE RBRACE s-plus)                         : (list)
         (LBRACE s-plus RBRACE)                         : (list)
	 (LBRACE s-plus RBRACE s-plus)                  : (list)
	 (LBRACE block-inner-plus RBRACE)               : $2
	 (LBRACE s-plus block-inner-plus RBRACE)        : $2
	 (LBRACE block-inner-plus RBRACE s-plus)        : $2
	 (LBRACE s-plus block-inner-plus RBRACE s-plus) : $2)
  (block-inner-plus (block-inner)                  : $1
		    (block-inner block-inner-plus) : (null-merge $1 $2))
  (block-inner (any)              : $1
	       (block)            : $1
	       (ATKEYWORD)        : $1
	       (ATKEYWORD s-plus) : $1
	       (s-semi)           : (list))
  
  ;; Ruleset parsing
  
  (ruleset-short-short 
   (RBRACE)                              : (list)
   (s-semi-decl-plus RBRACE)             : $1
   (declaration RBRACE)                  : $1
   (declaration s-semi-decl-plus RBRACE) : (cons $1 $2))
  (ruleset-short (LBRACE ruleset-short-short)        : $2
		 (LBRACE s-plus ruleset-short-short) : $3)

  (ruleset (ruleset-short)                 : (list (list (list "*")) $1)
	   (ruleset-short s-plus)          : (list (list (list "*")) $1)
	   (selector ruleset-short)        : 
	   (let ((m (merge-selectors $1)))
	     (begin (validate-selector m)
		    (list m $2)))
	   (selector ruleset-short s-plus) : 
	   (let ((m (merge-selectors $1)))
	     (begin (validate-selector m) 
		    (list m $2))))
  
  (selector (any)          : (list $1)
	    (any selector) : (if (member $1 '("." ":" "#"))
				 (cons (string-append $1 (car $2)) (cdr $2))
				 (null-merge $1 $2)))
  
  (declaration-short 
   (property DELIM value)               : (cons $1 (check-important $3))
   (property s-plus DELIM value)        : (cons $1 (check-important $4))
   (property DELIM s-plus value)        : (cons $1 (check-important $4))
   (property s-plus DELIM s-plus value) : (cons $1 (check-important $5)))
  (declaration (DELIM declaration-short) : $2
	       (declaration-short)       : $1)
  
  (property (IDENT) : $1)
  
  (block-or-s-semi (block)  : $1
		   (s-semi) : (list))
  (s-semi (SEMICOLON)        : (list)
	  (SEMICOLON s-plus) : (list))
  (s-semi-decl (s-semi)             : (list)
	       (s-semi declaration) : $2)
  (s-semi-decl-plus (s-semi-decl)                  : $1
		    (s-semi-decl s-semi-decl-plus) : (cons $1 $2))
  
  (value (single-value)       : $1
	 (single-value value) : (if (and (equal? $1 "!") 
					 (equal? $2 "important"))
				    "!important"
				    (string-append $1 " " $2)))
  (single-value (any)              : $1
		(block)            : $1
		(ATKEYWORD)        : $1
		(ATKEYWORD s-plus) : $1)
  
  (any-short (IDENT)                           : $1
	     (NUMBER)                          : $1
	     (PERCENTAGE)                      : $1
	     (DIMENSION)                       : $1
	     (STRING)                          : $1
	     (DELIM)                           : $1
	     (URI)                             : $1
	     (HASH)                            : $1
	     (UNICODE-RANGE)                   : $1
	     (EXACTMATCH)                      : "="
	     (INCLUDES)                        : "~="
	     (DASHMATCH)                       : "|="
	     
	     (FUNCTION RPAREN)                 : $1
	     (FUNCTION s-plus RPAREN)          : $1
	     (FUNCTION any-plus RPAREN)        : $1
	     (FUNCTION s-plus any-plus RPAREN) : $1
	     
	     (LPAREN RPAREN)                   : (list)
	     (LPAREN s-plus RPAREN)            : (list)
	     (LPAREN any-plus RPAREN)          : $2
	     (LPAREN s-plus any-plus RPAREN)   : $3
	     
	     (any-short LBRACK RBRACK)         : "[]"
	     (any-short LBRACK s-plus RBRACK)  : "[]"
	     (any-short LBRACK any-plus RBRACK) : 
	     (string-append $1 "[" (apply string-append $3) "]")
	     (any-short LBRACK s-plus any-plus RBRACK) :
	     (string-append $1 "[" (apply string-append $4) "]")
	     
	     (LBRACK RBRACK)                   : "[]"
	     (LBRACK s-plus RBRACK)            : "[]"
	     (LBRACK any-plus RBRACK)          : 
	     (string-append "[" (apply string-append $2) "]")
	     (LBRACK s-plus any-plus RBRACK)   : 
	     (string-append "[" (apply string-append $3) "]"))
  (any (any-short)        : $1
       (any-short s-plus) : $1)
  (any-plus (any)          : (list $1)
	    (any any-plus) : (null-merge $1 $2))
  
  (s-plus (S)        : (list)        
	  (S s-plus) : (list)))

